@@ -96,5 +96,5 @@ VALIDATION_ERRORS = {}
 VALIDATION_WARNINGS = {}
 
 # Toggle whether DataField instances should cache the underlying data
-# for their most common data access methods.
-DATA_CACHE_ENABLED = True
+# for their most common data access methods. 
+DATA_CACHE_ENABLED = False
