@@ -8,14 +8,14 @@ log = logging.getLogger(__file__)
 class Base(models.Model):
     """Base abstract class containing general metadata.
 
-        ``name`` - The name _should_ be unique in practice, but is not enforced
+        `name` - The name _should_ be unique in practice, but is not enforced
         since in certain cases the name differs relative to the model and/or
         concepts these fields are asssociated with.
 
-        ``description`` - Will tend to be exposed in client applications since
+        `description` - Will tend to be exposed in client applications since
         it provides context to the end-users.
 
-        ``keywords`` - Additional extraneous text that cannot be derived from
+        `keywords` - Additional extraneous text that cannot be derived from
         the name, description or data itself. This is solely used for search
         indexing.
     """
@@ -24,18 +24,9 @@ class Base(models.Model):
     description = models.TextField(null=True, blank=True)
     keywords = models.CharField(max_length=100, null=True, blank=True)
 
-    # Availability control mechanisms
-    # Rather than deleting objects, they can be archived
-    archived = models.BooleanField(default=False,
-        help_text=u'Note: archived takes precedence over being published')
-    # When `published` is false, it is globally not accessible.
-    published = models.BooleanField(default=False)
-
     created = models.DateTimeField(auto_now_add=True)
     modified = models.DateTimeField(auto_now=True)
 
-    objects = PublishedManager()
-
     class Meta(object):
         abstract = True
 
@@ -44,25 +35,21 @@ class Base(models.Model):
 
     @property
     def descriptors(self):
+        """Returns a dict of fields that represent this model's descriptors.
+        This is used for performing text indexing for model instances.
+        """
         return {
             'name': self.name,
             'description': self.description,
             'keywords': self.keywords,
         }
 
-    def save(self, *args, **kwargs):
-        if self.archived and self.published:
-            self.published = False
-            log.debug(u'{0} is published, but is being archived. It has ' \
-                'been unpublished'.format(self))
-        super(Base, self).save(*args, **kwargs)
-
 
 class BasePlural(Base):
     """Adds field for specifying the plural form of the name.
 
-        ``name_plural`` - Same as ``name``, but the plural form. If not
-        provided, an 's' will appended to the end of the ``name``.
+        `name_plural` - Same as `name`, but the plural form. If not
+        provided, an 's' will appended to the end of the `name`.
     """
     name_plural = models.CharField(max_length=200, null=True, blank=True)
 
@@ -86,3 +73,31 @@ class BasePlural(Base):
         else:
             plural = self.name
         return plural
+
+
+class PublishArchiveMixin(models.Model):
+    """Mixin containing the `published` and `archived` fields.
+
+    These fields are used in conjunction with the custom manager that is
+    defined for this model. The manager exposes a `published` method that
+    returns a queryset of objecs that are published and not archived.
+
+    If an object is saved that is published _and_ archived, the published
+    flag will be set to False.
+    """
+    published = models.BooleanField(default=False)
+
+    archived = models.BooleanField(default=False,
+        help_text=u'Note: archived takes precedence over being published')
+
+    objects = PublishedManager()
+
+    class Meta(object):
+        abstract = True
+
+    def save(self, *args, **kwargs):
+        if self.archived and self.published:
+            self.published = False
+            log.debug(u'{0} is published, but is being archived. It has ' \
+                'been unpublished'.format(self))
+        super(PublishArchiveMixin, self).save(*args, **kwargs)
