@@ -1,4 +1,5 @@
 import re
+from datetime import datetime
 from django.db import models
 from django.contrib.sites.models import Site
 from django.contrib.auth.models import User, Group
@@ -8,11 +9,11 @@ from django.db.models.fields import FieldDoesNotExist
 from django.db.models.signals import post_save, pre_delete
 from django.core.validators import RegexValidator
 from avocado.core import utils
-from avocado.core.models import Base, BasePlural
+from avocado.core.models import Base, BasePlural, PublishArchiveMixin
 from avocado.core.cache import (post_save_cache, pre_delete_uncache,
-    cached_property)
+    cached_method)
 from avocado.conf import settings
-from avocado import managers
+from avocado import managers, history
 from avocado.query.models import AbstractDataView, AbstractDataContext, AbstractDataQuery
 from avocado.query.translators import registry as translators
 from avocado.query.operators import registry as operators
@@ -20,23 +21,19 @@ from avocado.lexicon.models import Lexicon
 from avocado.sets.models import ObjectSet
 from avocado.events.models import Log
 from avocado.stats.agg import Aggregator
-from avocado.formatters import registry as formatters
-from avocado.queryview import registry as queryviews
+from avocado import formatters
 
 
 __all__ = ('DataCategory', 'DataConcept', 'DataField',
     'DataContext', 'DataView', 'DataQuery')
 
 
-SIMPLE_TYPE_MAP = settings.SIMPLE_TYPE_MAP
-
-
 ident_re = re.compile(r'^[a-zA-Z][a-zA-Z0-9_]*$')
 validate_ident = RegexValidator(ident_re, _("Enter a valid 'identifier' " \
     "that is a valid Python variable name."), 'invalid')
 
 
-class DataCategory(Base):
+class DataCategory(Base, PublishArchiveMixin):
     "A high-level organization for data concepts."
     # A reference to a parent for hierarchical categories
     parent = models.ForeignKey('self', null=True, blank=True,
@@ -51,7 +48,7 @@ class DataCategory(Base):
         verbose_name_plural = 'data categories'
 
 
-class DataField(BasePlural):
+class DataField(BasePlural, PublishArchiveMixin):
     """Describes the significance and/or meaning behind some data. In addition,
     it defines the natural key of the Django field that represents the location
     of that data e.g. ``library.book.title``.
@@ -88,12 +85,12 @@ class DataField(BasePlural):
     translator = models.CharField(max_length=100, blank=True, null=True,
         choices=translators.choices)
 
-    # The timestamp of the last time the underlying data for this field
-    # has been modified. This is used for the cache key and for general
-    # reference. The underlying table may not have a timestamp nor is it
-    # optimal to have to query the data for the max `modified` time.
-    data_modified = models.DateTimeField(null=True, help_text='The last time '\
-        ' the underlying data for this field was modified.')
+    # This is used for the cache key to check if the cached values is stale.
+    data_version = models.IntegerField(default=1, help_text='The current '\
+            'version of the underlying data for this field as of the last '\
+            'modification/update.')
+
+    group = models.ForeignKey(Group, null=True, blank=True, related_name='fields+')
 
     # Certain fields may not be relevant or appropriate for all
     # sites being deployed. This is primarily for preventing exposure of
@@ -102,8 +99,6 @@ class DataField(BasePlural):
     # has full access to all fields, while the external may have a limited set.
     # NOTE this is not reliable way to prevent exposure of sensitive data.
     # This should be used to simply hide _access_ to the concepts.
-    group = models.ForeignKey(Group, null=True, blank=True, related_name='fields+')
-
     sites = models.ManyToManyField(Site, blank=True, related_name='fields+')
 
     # The order of this datafield with respect to the category (if defined).
@@ -193,7 +188,7 @@ class DataField(BasePlural):
         By default, it will use the field's internal type, but can be
         overridden by the ``SIMPLE_TYPE_MAP`` setting.
         """
-        return SIMPLE_TYPE_MAP.get(self.internal_type, self.internal_type)
+        return utils.get_simple_type(self.field)
 
     @property
     def lexicon(self):
@@ -222,7 +217,6 @@ class DataField(BasePlural):
     # Convenience Methods
     # Easier access to the underlying data for this data field
 
-    @property
     def values_list(self):
         "Returns a `ValuesListQuerySet` of values for this field."
         if self.lexicon or self.objectset:
@@ -238,7 +232,7 @@ class DataField(BasePlural):
             else:
                 field_name = self.field_name
             filters = {u'{0}__icontains'.format(field_name): query}
-            return self.values_list.filter(**filters)
+            return self.values_list().filter(**filters)
 
     def get_plural_unit(self):
         if self.unit_plural:
@@ -262,22 +256,21 @@ class DataField(BasePlural):
                 return self.model.objects.filter(**kwargs)\
                     .values_list('name', flat=True)[0]
             return smart_unicode(value)
-        return dict(self.choices).get(value, smart_unicode(value))
+        return dict(self.choices()).get(value, smart_unicode(value))
 
     # Data-related Cached Properties
     # These may be cached until the underlying data changes
-
-    @cached_property('size', version='data_modified')
+    @cached_method(version='data_version')
     def size(self):
         "Returns the count of distinct values."
-        return self.values_list.count()
+        return self.values_list().count()
 
-    @cached_property('values', version='data_modified')
+    @cached_method(version='data_version')
     def values(self):
         "Returns a distinct list of the values."
-        return tuple(self.values_list)
+        return tuple(self.values_list())
 
-    @cached_property('labels', version='data_modified')
+    @cached_method(version='data_version')
     def labels(self):
         """Returns an ordered set of labels corresponding to the values.
         If this field represents to a Lexicon subclass, the `label` field
@@ -289,69 +282,72 @@ class DataField(BasePlural):
             return tuple(self.model.objects.values_list('name', flat=True))
         # Unicode each value, use an iterator here to prevent loading the
         # raw values in memory
-        return map(smart_unicode, iter(self.values_list))
+        return map(smart_unicode, iter(self.values_list()))
 
-    @cached_property('codes', version='data_modified')
+    @cached_method(version='data_version')
     def codes(self):
         "Returns a distinct set of coded values for this field"
         if self.lexicon:
             return tuple(self.model.objects.values_list('code', flat=True))
 
-    @property
     def choices(self):
         "Returns a distinct set of choices for this field."
-        return zip(self.values, self.labels)
+        return zip(self.values(), self.labels())
 
-    @property
     def coded_choices(self):
         "Returns a distinct set of coded choices for this field."
         if self.lexicon:
-            return zip(self.codes, self.labels)
+            return zip(self.codes(), self.labels())
 
-    @property
     def coded_values(self):
         "Returns a distinct set of coded values for this field."
         if self.lexicon:
-            return zip(self.values, self.codes)
+            return zip(self.values(), self.codes())
 
     # Data Aggregation Properties
     def groupby(self, *args):
         return Aggregator(self.field).groupby(*args)
 
+    @cached_method(version='data_version')
     def count(self, *args, **kwargs):
         "Returns an the aggregated counts."
         return Aggregator(self.field).count(*args, **kwargs)
 
+    @cached_method(version='data_version')
     def max(self, *args):
         "Returns the maximum value."
         return Aggregator(self.field).max(*args)
 
+    @cached_method(version='data_version')
     def min(self, *args):
         "Returns the minimum value."
         return Aggregator(self.field).min(*args)
 
+    @cached_method(version='data_version')
     def avg(self, *args):
         "Returns the average value. Only applies to quantitative data."
         if self.simple_type == 'number':
             return Aggregator(self.field).avg(*args)
 
+    @cached_method(version='data_version')
     def sum(self, *args):
         "Returns the sum of values. Only applies to quantitative data."
         if self.simple_type == 'number':
             return Aggregator(self.field).sum(*args)
 
+    @cached_method(version='data_version')
     def stddev(self, *args):
         "Returns the standard deviation. Only applies to quantitative data."
         if self.simple_type == 'number':
             return Aggregator(self.field).stddev(*args)
 
+    @cached_method(version='data_version')
     def variance(self, *args):
         "Returns the variance. Only applies to quantitative data."
         if self.simple_type == 'number':
             return Aggregator(self.field).variance(*args)
 
     # Translator Convenience Methods
-
     @property
     def operators(self):
         "Returns the valid operators for this datafield."
@@ -369,7 +365,7 @@ class DataField(BasePlural):
         return trans.validate(self, operator, value, tree, **context)
 
 
-class DataConcept(BasePlural):
+class DataConcept(BasePlural, PublishArchiveMixin):
     """Our acceptance of an ontology is, I think, similar in principle to our
     acceptance of a scientific theory, say a system of physics; we adopt, at
     least insofar as we are reasonable, the simplest conceptual scheme into
@@ -399,6 +395,9 @@ class DataConcept(BasePlural):
     fields = models.ManyToManyField(DataField, through='DataConceptField',
         related_name='concepts')
 
+    group = models.ForeignKey(Group, null=True, blank=True,
+        related_name='concepts+')
+
     # Certain concepts may not be relevant or appropriate for all
     # sites being deployed. This is primarily for preventing exposure of
     # access to private data from certain sites. For example, there may and
@@ -406,22 +405,24 @@ class DataConcept(BasePlural):
     # has full access to all fields, while the external may have a limited set.
     # NOTE this is not reliable way to prevent exposure of sensitive data.
     # This should be used to simply hide _access_ to the concepts.
-    group = models.ForeignKey(Group, null=True, blank=True,
-        related_name='concepts+')
-
     sites = models.ManyToManyField(Site, blank=True, related_name='concepts+')
 
     order = models.FloatField(null=True, blank=True, db_column='_order')
 
     # An optional formatter which provides custom formatting for this
-    # concept relative to the associated fields. If a formatter is not
-    # defined, this DataConcept is not intended to be exposed since the
-    # underlying data may not be appropriate for client consumption.
+    # concept relative to the associated fields.
     formatter_name = models.CharField('formatter', max_length=100, blank=True,
-        null=True, choices=formatters.choices)
+        null=True, choices=formatters.registry.choices)
 
-    queryview = models.CharField(max_length=100, blank=True, null=True,
-        choices=queryviews.choices)
+    # A flag that denotes this concept is viewable, that is, this the concept
+    # is appropriate to be used as a viewable interface. Non-viewable concepts
+    # can be used to prevent exposing underlying data that may not be
+    # appropriate for client consumption.
+    viewable = models.BooleanField(default=True)
+
+    # A flag that denotes this concept is 'queryable' which assumes fields
+    # that DO NOT result in a nonsensicle representation of the concept.
+    queryable = models.BooleanField(default=True)
 
     # A flag that denotes when this concept can be applied to an ORDER BY
     # Certain concepts are not appropriate because they are too complicated,
@@ -440,7 +441,7 @@ class DataConcept(BasePlural):
         name = self.formatter_name
         cache = getattr(self, '_formatter_cache', None)
         if not cache or name != cache[0]:
-            formatter = formatters.get(name)(self)
+            formatter = formatters.registry.get(name)(self)
             self._formatter_cache = (name, formatter)
         else:
             formatter = cache[1]
@@ -486,11 +487,16 @@ class DataContext(AbstractDataContext, Base):
     template = models.BooleanField(default=False)
     default = models.BooleanField(default=False)
 
+    # The parent this instance was derived from
+    parent = models.ForeignKey('self', null=True, blank=True,
+        related_name='forks')
+
     # For authenticated users the `user` can be directly referenced,
     # otherwise the session key can be used.
     user = models.ForeignKey(User, null=True, blank=True, related_name='datacontext+')
     session_key = models.CharField(max_length=40, null=True, blank=True)
 
+    accessed = models.DateTimeField(default=datetime.now(), editable=False)
     objects = managers.DataContextManager()
 
     def __unicode__(self):
@@ -515,22 +521,11 @@ class DataContext(AbstractDataContext, Base):
             toks.append('template')
         elif self.session:
             toks.append('session')
-        elif self.archived:
-            toks.append('archived')
         else:
             toks.append('rogue')
 
         return u'{0} ({1})'.format(*toks)
 
-    def archive(self):
-        if self.archived:
-            return False
-        backup = self.pk, self.session, self.archived
-        self.pk, self.session, self.archived = None, False, True
-        self.save()
-        self.pk, self.session, self.archived = backup
-        return True
-
     def clean(self):
         from django.core.exceptions import ValidationError
         if self.template and self.default:
@@ -555,11 +550,16 @@ class DataView(AbstractDataView, Base):
     template = models.BooleanField(default=False)
     default = models.BooleanField(default=False)
 
+    # The parent this instance was derived from
+    parent = models.ForeignKey('self', null=True, blank=True,
+        related_name='forks')
+
     # For authenticated users the `user` can be directly referenced,
     # otherwise the session key can be used.
     user = models.ForeignKey(User, null=True, blank=True, related_name='dataview+')
     session_key = models.CharField(max_length=40, null=True, blank=True)
 
+    accessed = models.DateTimeField(default=datetime.now(), editable=False)
     objects = managers.DataViewManager()
 
     def __unicode__(self):
@@ -584,22 +584,11 @@ class DataView(AbstractDataView, Base):
             toks.append('template')
         elif self.session:
             toks.append('session')
-        elif self.archived:
-            toks.append('archived')
         else:
             toks.append('rogue')
 
         return u'{0} ({1})'.format(*toks)
 
-    def archive(self):
-        if self.archived:
-            return False
-        backup = self.pk, self.session, self.archived
-        self.pk, self.session, self.archived = None, False, True
-        self.save()
-        self.pk, self.session, self.archived = backup
-        return True
-
     def clean(self):
         from django.core.exceptions import ValidationError
         if self.template and self.default:
@@ -616,10 +605,10 @@ class DataView(AbstractDataView, Base):
 
 class DataQuery(AbstractDataQuery, Base):
     """
-    JSON object representing a complete query. 
+    JSON object representing a complete query.
 
     The query is constructed from a context(providing the 'WHERE' statements)
-    and a view(providing the 'SELECT' and 'ORDER BY" statements). This 
+    and a view(providing the 'SELECT' and 'ORDER BY" statements). This
     corresponds to all the statements of the SQL query to dictate what info
     to retrieve, how to filter it, and the order to display it in.
     """
@@ -627,12 +616,18 @@ class DataQuery(AbstractDataQuery, Base):
     template = models.BooleanField(default=False)
     default = models.BooleanField(default=False)
 
+    # The parent this instance was derived from
+    parent = models.ForeignKey('self', null=True, blank=True,
+        related_name='forks')
+
     # For authenticated users the `user` can be directly referenced,
     # otherwise the session key can be used.
     user = models.ForeignKey(User, null=True, blank=True, related_name='dataquery+')
     session_key = models.CharField(max_length=40, null=True, blank=True)
 
+    accessed = models.DateTimeField(default=datetime.now, editable=False)
     objects = managers.DataQueryManager()
+    shared_users = models.ManyToManyField(User, related_name='shareddataquery+')
 
     def __unicode__(self):
         toks = []
@@ -656,22 +651,11 @@ class DataQuery(AbstractDataQuery, Base):
             toks.append('template')
         elif self.session:
             toks.append('session')
-        elif self.archived:
-            toks.append('archived')
         else:
             toks.append('rogue')
 
         return u'{0} ({1})'.format(*toks)
 
-    def archive(self):
-        if self.archived:
-            return False
-        backup = self.pk, self.session, self.archived
-        self.pk, self.session, self.archived = None, False, True
-        self.save()
-        self.pk, self.session, self.archived = backup
-        return True
-
     def clean(self):
         from django.core.exceptions import ValidationError
         if self.template and self.default:
@@ -686,6 +670,33 @@ class DataQuery(AbstractDataQuery, Base):
         super(self.__class__, self).save(*args, **kwargs)
 
 
+    def share_with_user(self, email, create_user=True):
+        """
+        Attempts to add a user with the supplied email address to the list of
+        shared users for this query. If create_user is set to True, users will
+        be created for emails that are not already associated with an
+        existing user.
+
+        Returns True if the email was added to the list of shared users and
+        False if the email wasn't added because it already exists or wasn't
+        created.
+        """
+        # If the query is already shared then there is no need to share it
+        # again.
+        if self.shared_users.filter(email=email).exists():
+            return False
+
+        try:
+            user = User.objects.get(email=email)
+        except User.DoesNotExist:
+            if not create_user:
+                return False
+            else:
+                user = utils.create_email_based_user(email)
+
+        self.shared_users.add(user)
+        self.save()
+
 # Register instance-level cache invalidation handlers
 post_save.connect(post_save_cache, sender=DataField)
 post_save.connect(post_save_cache, sender=DataConcept)
@@ -694,3 +705,9 @@ post_save.connect(post_save_cache, sender=DataCategory)
 pre_delete.connect(pre_delete_uncache, sender=DataField)
 pre_delete.connect(pre_delete_uncache, sender=DataConcept)
 pre_delete.connect(pre_delete_uncache, sender=DataCategory)
+
+# Register with history API
+if settings.HISTORY_ENABLED:
+    history.register(DataContext, fields=('name', 'description', 'json'))
+    history.register(DataView, fields=('name', 'description', 'json'))
+    history.register(DataQuery, fields=('name', 'description', 'context_json', 'view_json'))
