@@ -1,7 +1,11 @@
 import jsonfield
 from django.db import models
 from modeltree.tree import trees
-from . import parsers
+from . import oldparsers as parsers
+
+def _sql_string(queryset):
+    sql, params = queryset.query.sql_with_params()
+    return sql % tuple([repr(str(x)) for x in params])
 
 
 class AbstractDataContext(models.Model):
@@ -74,6 +78,13 @@ class AbstractDataContext(models.Model):
     def language(self, tree=None, **context):
         return self.parse(tree=tree, **context).language
 
+    def sql(self, *args, **kwargs):
+        """Returns the SQL query string representative of this context.
+
+        This takes the same arguments as `apply()`.
+        """
+        return _sql_string(self.apply(*args, **kwargs))
+
 
 class AbstractDataView(models.Model):
     """JSON object representing one or more data field conditions. The data may
@@ -83,7 +94,6 @@ class AbstractDataView(models.Model):
     """
     json = jsonfield.JSONField(null=True, blank=True, default=dict,
         validators=[parsers.dataview.validate])
-    count = models.IntegerField(null=True, db_column='_count')
 
     def __init__(self, *args, **kwargs):
         if args and isinstance(args[0], dict):
@@ -112,13 +122,20 @@ class AbstractDataView(models.Model):
         return self.parse(tree=tree, **context).apply(queryset=queryset,
             include_pk=include_pk)
 
+    def sql(self, *args, **kwargs):
+        """Returns the SQL query string representative of this view.
+
+        This takes the same arguments as `apply()`.
+        """
+        return _sql_string(self.apply(*args, **kwargs))
+
 
 class AbstractDataQuery(models.Model):
     """
-    JSON object representing a complete query. 
+    JSON object representing a complete query.
 
     The query is constructed from a context(providing the 'WHERE' statements)
-    and a view(providing the 'SELECT' and 'ORDER BY" statements). This 
+    and a view(providing the 'SELECT' and 'ORDER BY" statements). This
     corresponds to all the statements of the SQL query to dictate what info
     to retrieve, how to filter it, and the order to display it in.
     """
@@ -127,6 +144,11 @@ class AbstractDataQuery(models.Model):
     view_json = jsonfield.JSONField(null=True, blank=True, default=dict,
             validators=[parsers.dataview.validate])
 
+    # The count when just the context is applied
+    distinct_count = models.IntegerField(null=True)
+    # The count when the context and the view is applied
+    record_count = models.IntegerField(null=True)
+
     def __init__(self, *args, **kwargs):
         if args and isinstance(args[0], dict):
             if 'context_json' in kwargs:
@@ -142,6 +164,7 @@ class AbstractDataQuery(models.Model):
 
     class Meta(object):
         abstract = True
+        verbose_name_plural = 'data queries'
 
     @property
     def context(self):
@@ -169,12 +192,18 @@ class AbstractDataQuery(models.Model):
             'context': self.context_json,
             'view': self.view_json,
         }
-
         return parsers.dataquery.parse(json, tree=tree, **context)
 
     def apply(self, queryset=None, tree=None, distinct=True, include_pk=True, **context):
         "Applies this context to a QuerySet."
         if tree is None and queryset is not None:
             tree = queryset.model
-        return self.parse(tree=tree, **context).apply(queryset=queryset, 
+        return self.parse(tree=tree, **context).apply(queryset=queryset,
             distinct=distinct, include_pk=include_pk)
+
+    def sql(self, *args, **kwargs):
+        """Returns the SQL query string representative of this query.
+
+        This takes the same arguments as `apply()`.
+        """
+        return _sql_string(self.apply(*args, **kwargs))
